package com.simuwang.daq.components.report.parser.pdf;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.map.MapUtil;
import com.simuwang.base.common.exception.APIException;
import com.simuwang.base.mapper.EmailFieldMappingMapper;
import com.simuwang.base.pojo.dto.report.MonthlyReportData;
import com.simuwang.base.pojo.dto.report.ReportFundInfoDTO;
import com.simuwang.base.pojo.dto.report.ReportNetReportDTO;
import com.simuwang.base.pojo.dto.report.ReportParserParams;
import com.simuwang.daq.components.report.parser.ReportParserConstant;
import org.springframework.stereotype.Component;
import technology.tabula.RectangularTextContainer;
import technology.tabula.Table;

import java.util.List;
import java.util.Map;

/**
 * @author wangzaijun
 * @date 2024/9/11 16:19
 * @description pdf格式的月报解析
 */
@Component(ReportParserConstant.PARSER_PDF_MONTHLY)
public class PDMonthlyReportParser extends AbstractPDReportParser<MonthlyReportData> {
    private final List<Table> extNavTables = ListUtil.list(true);
    private Table fundInfoTable = null;

    public PDMonthlyReportParser(EmailFieldMappingMapper fieldMappingMapper) {
        super(fieldMappingMapper);
    }

    @Override
    public String getParser() {
        return ReportParserConstant.PARSER_PDF_MONTHLY;
    }

    @Override
    protected void initTableInfo(List<Table> tables) {
        // 一般月报是固定的模板，4列表格是基金基本信息，其他5列的表格是月净值
        for (Table table : tables) {
            int colCount = table.getColCount();
            if (colCount == 4) {
                this.fundInfoTable = table;
            } else if (colCount >= 5) {
                this.extNavTables.add(table);
            }
        }
    }

    @Override
    protected ReportFundInfoDTO parseFundInfo(ReportParserParams params) {
        Table fundInfoTable = this.fundInfoTable;
        if (fundInfoTable == null) {
            throw new APIException("未解析到基本信息表格");
        }
        Map<String, Object> baseInfoMap = MapUtil.newHashMap(32);
        for (int i = 0; i < fundInfoTable.getRows().size(); i++) {
            @SuppressWarnings("all")
            List<RectangularTextContainer> cols = fundInfoTable.getRows().get(i);
            for (int j = 0; j < 2; j++) {
                baseInfoMap.put(cols.get(j * 2).getText(), cols.get(j * 2 + 1).getText());
            }
        }
        // 匹配字段清洗字段
        ReportFundInfoDTO reportFundInfo = new ReportFundInfoDTO();
        reportFundInfo.setFileId(params.getFileId());
        this.buildInfo(baseInfoMap, reportFundInfo);
        return reportFundInfo;
    }

    @Override
    protected MonthlyReportData parseExtInfoAndSetData(String reportName, ReportFundInfoDTO fundInfo, ReportParserParams params) {
        Integer fileId = params.getFileId();
        MonthlyReportData reportData = new MonthlyReportData();
        reportData.setBaseInfo(this.buildReportInfo(fileId, reportName));
        reportData.setFundInfo(fundInfo);
        // 母基金和分级基金的净值
        List<ReportNetReportDTO> exts = ListUtil.list(false);
        List<Table> extNavTables = this.extNavTables;
        for (Table extNavTable : extNavTables) {
            Map<String, Object> extInfoMap = MapUtil.newHashMap(16);
            for (int i = 0; i < extNavTable.getColCount(); i++) {
                String key = extNavTable.getCell(0, i).getText();
                String value = extNavTable.getCell(1, i).getText();
                extInfoMap.put(key, value);
            }
            ReportNetReportDTO navInfo = new ReportNetReportDTO();
            navInfo.setFileId(fileId);
            this.buildInfo(extInfoMap, navInfo);
            exts.add(navInfo);
        }
        // 分级基金匹配
        List<String> levels = this.matchTieredFund(String.join(",", this.textList));
        levels.add(0, "母基金");
        for (int i = 0; i < exts.size(); i++) {
            if (levels.size() <= i) {
                continue;
            }
            exts.get(i).setLevel(levels.get(i));
        }
        reportData.setNetReport(exts);
        return reportData;
    }

    @Override
    protected void cleaningReportData(MonthlyReportData reportData) {
        // todo 数据清洗
    }
}
